/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.views;

import static org.fortiss.tooling.kernel.extension.data.IConstraintViolation.ESeverity.DEBUG;
import static org.fortiss.tooling.kernel.extension.data.IConstraintViolation.ESeverity.ERROR;
import static org.fortiss.tooling.kernel.extension.data.IConstraintViolation.ESeverity.FATAL;
import static org.fortiss.tooling.kernel.extension.data.IConstraintViolation.ESeverity.INFO;
import static org.fortiss.tooling.kernel.extension.data.IConstraintViolation.ESeverity.WARNING;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;
import org.fortiss.tooling.kernel.extension.data.IConstraintViolation;
import org.fortiss.tooling.kernel.extension.data.IConstraintViolation.ESeverity;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.ui.extension.IModelElementHandler;
import org.fortiss.tooling.kernel.ui.internal.MarkerService;
import org.fortiss.tooling.kernel.ui.service.IMarkerService;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;

/**
 * {@link ViewPart} for the marker display.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 360983B847696BC72FE50E894CCAC89A
 */
public class MarkerViewPart extends ViewPart {

	/** GUI builder viewer instance. */
	private MarkerViewerGUI gui;

	/** Constructor. */
	public MarkerViewPart() {
		((MarkerService) IMarkerService.INSTANCE).setMarkerViewPart(this);
	}

	/** {@inheritDoc} */
	@Override
	@SuppressWarnings("unchecked")
	public void createPartControl(Composite parent) {
		parent.setLayout(new FillLayout());
		gui = new MarkerViewerGUI(parent, SWT.NONE);
		gui.getTreeViewer().setContentProvider(new ITreeContentProvider() {

			@Override
			public void inputChanged(Viewer viewer, Object oldInput,
					Object newInput) {
				// ignore
			}

			@Override
			public void dispose() {
				// do nothing
			}

			@Override
			public boolean hasChildren(Object element) {
				Object[] objs = getChildren(element);
				return objs != null && objs.length > 0;
			}

			@Override
			public Object getParent(Object element) {
				if (element instanceof IConstraintViolation<?>) {
					return ((IConstraintViolation<?>) element).getSeverity();
				}
				return null;
			}

			@Override
			public Object[] getElements(Object inputElement) {
				return new Object[] { FATAL, ERROR, WARNING, INFO, DEBUG };
			}

			@Override
			public Object[] getChildren(Object parentElement) {
				if (parentElement instanceof ESeverity) {
					return IMarkerService.INSTANCE
							.getAllViolationsWithSeverity(
									(ESeverity) parentElement).toArray();
				}
				return null;
			}
		});
		gui.getSeverityColumn().setLabelProvider(new CellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				if (cell.getElement() instanceof ESeverity) {
					cell.setText(((ESeverity) cell.getElement()).toString());
				} else {
					cell.setText("");
				}
			}
		});
		gui.getElementColumn().setLabelProvider(new CellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				if (cell.getElement() instanceof IConstraintViolation) {
					IConstraintViolation<? extends EObject> violation = (IConstraintViolation<? extends EObject>) cell
							.getElement();
					IModelElementHandler<EObject> handler = IModelElementHandlerService.INSTANCE
							.getModelElementHandler(violation.getSource());
					cell.setText(handler != null ? handler.getName(violation
							.getSource()) : "");
				} else {
					cell.setText("");
				}
			}
		});
		gui.getExplanationColumn().setLabelProvider(new CellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				if (cell.getElement() instanceof IConstraintViolation) {
					IConstraintViolation<? extends EObject> violation = (IConstraintViolation<? extends EObject>) cell
							.getElement();
					cell.setText(violation.getExplanation());
				} else {
					cell.setText("");
				}
			}
		});
		gui.getProjectColumn().setLabelProvider(new CellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				if (cell.getElement() instanceof IConstraintViolation) {
					IConstraintViolation<? extends EObject> violation = (IConstraintViolation<? extends EObject>) cell
							.getElement();
					EObject rootElement = IPersistencyService.INSTANCE
							.getTopLevelElementFor(violation.getSource())
							.getRootModelElement();
					IModelElementHandler<EObject> handler = IModelElementHandlerService.INSTANCE
							.getModelElementHandler(rootElement);
					cell.setText(handler != null ? handler.getName(rootElement)
							: "");
				} else {
					cell.setText("");
				}
			}
		});

		gui.getTreeViewer().setInput(IMarkerService.INSTANCE);
	}

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		gui.getTreeViewer().getControl().setFocus();
	}

	/** Refreshes the view. Needs to be called on UI thread. */
	public void refresh() {
		gui.getTreeViewer().refresh();
	}
}
